;**************************************************************************
;*
;* FILE  fmt2_1541.asm
;* Written by Daniel Kahlin <daniel@kahlin.net>
;* $Id: fmt2_1541.asm,v 1.44 2010-03-31 19:58:55 tlr Exp $
;*
;* DESCRIPTION
;*   Fast format routine for the 1541/1571 (drive part).
;*
;* REFERENCES
;*   "Inside Commodore DOS", Richard Immers and Gerald G. Neufeld,
;*        Second Printing 1985, ISBN 0-88190-366-3, Datamost, Inc.
;*   "Das grosse FloppyBuch", Englisch & Szczepanowski, 1984, 
;*        ISBN 3-89011-005-3, Data Becker,GmbH 
;*
;******
	processor 6502
	
	include "macros.i"

SKIP_BAM_WRITE		equ	0
MEASURE_TAIL_GAP	equ	1
MEASURE_TAIL_AVERAGE	equ	0
	
STEP_TIMER	equ	$3a ; 15 ms
TIMER_10MS	equ	$28 ; 10 ms
SPIN_UP_TIME	equ	50  ; * 10 ms = 500 ms

FMT_TRACKS	equ	35

; $2100 should be safe down to 274 RPM
; (60/274)*(1000000/26) = 8422.2
MAX_TRACK_CAPACITY equ	$2100

; 5 + 10 + 9 + 5 + 325 = $0162, but the 1541 ROM uses $0166...
SECTOR_GCR_SPACE	equ	$0166


FMT_HDRBUFFER	equ	$0300
fmt_sectortemp	equ	$71	;$0628
fmt_retries	equ	$ba	;$0620
fmt_vfyretries	equ	$b5	;$0623
fmt_intersectorgap equ	$b6	;$0626
fmt_capacity	equ	$fe	; check for a free spot for this.
fmt_zone	equ	$fd	; current zone
	
tmp_zp		equ	$73
track_zp	equ	$22
currhalftrk_zp	equ	$72
.shift_zp	equ	$73

	org	$0300
format_code:
	jmp	format_code_entry

;**************************************************************************
;*
;* main format routine...
;* started by an execute job buffer 1.
;* just to put us running in FDC mode.
;*
;******
formatjob:
	jsr	motor_on

	lda	#%11101110
	sta	$1c0c

; handle if we are positioned on a half track
	lda	$1c00
	lsr
	bcc	wce_skp1
	jsr	head_in
wce_skp1:
; step in (higher track), then out (lower track) to settle head
; This is because some drive mechanisms will feel bad when going
; directly from step 10 -> step 00 (i.e track 18 -> reset routine)	
	jsr	head_in		
 	jsr	head_out

; scan for valid sector headers to determine which track we are actually
; on!

	lda	#$3f
	sta	$4b
rce_lp1:
	lda	$4b
	and	#7
	bne	rce_skp1
; change the density every 8 retries
	lda	$1c00
	clc
	adc	#%00100000
;	and	#%01111111	; sync bit doesn't matter.
	sta	$1c00
rce_skp1:

	lda	#$80 | $50	; 20 ms
	sta	$1805
rce_lp2:
	bit	$1805		; timeout?
	bpl	rce_fl1		; yes, fail exit
	bit	$1c00		; Sync found?
	bmi	rce_lp2		; no, try again
	lda	$1c01
	clv			; flush byte
	ldx	#0
rce_lp3:
	bvc	.
	clv
	lda	$1c01
	sta	$24,x
	inx
	cpx	#8
	bne	rce_lp3
	lda	$24
	cmp	#$52
	bne	rce_fl1
	jsr	$f497		; decode
	lda	$16
	eor	$17
	eor	$18
	eor	$19
	eor	$1a
	beq	rce_found
rce_fl1:
	dec	$4b
	bpl	rce_lp1

; failed, go to error state
	jmp	rce_fl2
	
rce_found:
	lda	$18
	sta	$22

rce_fl2:

;*
;* Determine how many tracks to bump.
;* If we know the current track, just step in all tracks plus ~4 extra.
;* Otherwise (if DRVTRK=0) do normal bump.
;* Always make sure step bits in $1c00 become 00.
;*
	lda	$22	;DRVTRK
	bne	fj_skp2
	lda	#44
fj_skp2:
	asl		;A=A*2
	clc
	adc	#4
	tay

fj_llp1:
	jsr	head_out
	dey
	bne	fj_llp1

	lda	$1c00
	and	#%00000011
	beq	fj_skpo1
	tay
fj_llp2:
	jsr	head_out
	dey
	bne	fj_llp1
fj_skpo1:	
;*
;* Setup track
;*
	lda	#1
	sta	$22	;DRVTRK
	sta	$51	;FTNUM
	asl
	sta	currhalftrk_zp

	lda	#10
	sta	fmt_retries

;**************************************************************************
;*
;* prepare and check write protect
;*
;******
	lda	#%00010000
	bit	$1c00
	bne	fj_skpo2
	lda	#$08	;26, 'write protect on'
	rts	; jmp $fddb
fj_skpo2:

;**************************************************************************
;*
;* measure track
;*
;******
MEASURE_OFFSET	equ	$10000-MAX_TRACK_CAPACITY
fj_measure:
	jsr	set_density

	jsr	start_wipe
	ldx	#>MAX_TRACK_CAPACITY
	ldy	#<MAX_TRACK_CAPACITY
fjm_lp1:
	bvc	.
	clv
	dey
	bne	fjm_lp1
	dex
	bne	fjm_lp1

	ldx	#>MEASURE_OFFSET	; Start at offset to time out earlier.
	lda	#$ff
	sta	$1c01
fjm_lp2:
	bvc	.
	clv
	iny
	cpy	#$05
	bne	fjm_lp2
	jsr	$fe00		; switch to read mode
fjm_lp3:
	lda	$1c00
	bpl	fjm_skp1
	bvc	fjm_lp3
	clv
	iny
	bne	fjm_lp3
	inx
	bne	fjm_lp3

	lda	#$02		; $02  Header block not found

fjm_fl1:
;* measure error, decrease retries
	dec	fmt_retries
	bne	fj_measure
	rts	; jmp $fddb

;SCALE_ZONE0	equ	65536	; 65536*26/26
SCALE_ZONE1	equ	60854	; 65536*26/28
SCALE_ZONE2	equ	56797	; 65536*26/30
SCALE_ZONE3	equ	53248	; 65536*26/32
mull_tab:
	dc.b	<SCALE_ZONE1,<SCALE_ZONE2,<SCALE_ZONE3
mulh_tab:
	dc.b	>SCALE_ZONE1,>SCALE_ZONE2,>SCALE_ZONE3

fjm_skp1:
; Y/X = Capacity + MEASURE_OFFSET
	sty	capl_tab
	txa
	sec
	sbc	#>MEASURE_OFFSET
	sta	caph_tab

; calculate capacities for all zones by scaling with constants
; capl_tab and caph_tab must be all zeroes.
	ldx	#2
loop:
	lda	capl_tab
	sta	fmt_capacity
	lda	caph_tab
	sta	fmt_capacity+1

;* cap[l|h]_tab = (fmt_capacity * mul[l|h]_tab) / 65536
	ldy	#16		; set binary count to 16 
shift_r:
	lsr	fmt_capacity+1	; divide multiplier by 2 
	ror	fmt_capacity
	bcc	rotate_r
	lda	capl_tab+1,x	; get upper half of product and add multiplicand
	clc
	adc	mull_tab,x
	sta	capl_tab+1,x
	lda	caph_tab+1,x
	adc	mulh_tab,x
rotate_r:
	ror	;rotate partial product 
	sta	caph_tab+1,x
	ror	capl_tab+1,x
	dey
	bne	shift_r

; go to next zone
	dex
	bpl	loop

	ldy	#3
	sty	fmt_zone
fjm_lp10:
	ldy	fmt_zone
	lda	sec_tab,y
	sta	$43	; SECTR
	lda	capl_tab,y
	ldx	caph_tab,y
;**************************************************************************
;*
;* calculate intersector gaps
;*
;******
	ldy	$43	; SECTR
fjm_lp4:
	sec
	sbc	conf_sector_gcr_space
	bcs	fjm_skp2
	dex
fjm_skp2:
	dex	;>SECTOR_GCR_SPACE == 1 
	dey
	bne	fjm_lp4
; Acc/X = Capacity - SECTOR_GCR_SPACE * num_sectors

	
; Y=Acc/X / num_sectors
	ldy	#0
fjm_lp5:
	sec
	sbc	$43	; SECTR
	bcs	fjm_skp3
	dex
	bmi	fjm_skp4
fjm_skp3:
	iny
	bne	fjm_lp5		; always taken
fjm_skp4:
; Y=intersector_gap
	tya
	ldx	fmt_zone
	sta	isgap_tab,x
	
	cpy	#4		; gap less than 4?
	bcs	fjm_skp5	; no, great then we format!

	lda	#$05		; $05  Checksum error in data block
	jmp	fjm_fl1

fjm_skp5:
	dec	fmt_zone
	bpl	fjm_lp10

	if	. < $0400
	ERROR	"main_loop overlap!"
	endif
	echo	"main_loop",.
;**************************************************************************
;*
;* main_loop
;*
;******
main_loop:
	lda	$51
	sta	last_track
	jsr	seek_track
	stx	last_zone

;**************************************************************************
;*
;* format a track
;*
;******
fj_format:
	lda	isgap_tab,x
	sta	fmt_intersectorgap
;*
;* start wipeing track with $55's
;* this happens automagically once started.
;*
	jsr	start_wipe

;*
;* calculate sectorheaders (while still wipeing track)
;*
	ldy	#0
	ldx	#0
fj_lp2:
	lda	$39	;HBID (always $08)
	sta	FMT_HDRBUFFER,y
	txa		;SECTOR
	sta	FMT_HDRBUFFER+2,y
	eor	$51	;FTNUM
	eor	$13	;DSKID2 
	eor	$12	;DSKID1
	sta	FMT_HDRBUFFER+1,y	;CHKSUM
	lda	$51	;FTNUM
	sta	FMT_HDRBUFFER+3,y
	lda	$13	;DSKID2
	sta	FMT_HDRBUFFER+4,y
	lda	$12	;DSKID1
	sta	FMT_HDRBUFFER+5,y
	lda	#$0f
	sta	FMT_HDRBUFFER+6,y
	sta	FMT_HDRBUFFER+7,y

	tya
	clc
	adc	#8
	tay
	
	inx
	cpx	$43	;SECTR
	bne	fj_lp2
	stx	fmt_sectortemp
;*
;* GCR code headers (while still wipeing track)
;*
	tya
	pha
	lda	#>FMT_HDRBUFFER
	sta	$31	;BUFPNT+1
 	jsr	$fe30	;FBTOG  (always sets BUFPNT to 0)
	pla
	tay
	dey
 	jsr	$fde5	;MOVUP
 	jsr	$fdf5	;MOVOVR

;**************************************************************************
;*
;* write the track.
;*
;******
	lda	#0
	sta	$32	;HDRPNT
wrt_lp1:

; write header block sync
	jsr	writesync

; write header block
	ldy	$32	;HDRPNT
	ldx	#10
	START_SAMEPAGE "write1"
wrt_lp2:
	bvc	.
	clv
	lda	FMT_HDRBUFFER,y
	sta	$1c01
	iny
	dex
	bne	wrt_lp2
	END_SAMEPAGE
	sty	$32	;HDRPNT

; write header gap
	lda	#$55
	ldx	#9
wrt_lp3:
	bvc	.
	clv
	sta	$1c01
	dex
	bne	wrt_lp3

; write data block sync
	jsr	writesync

; write data block
	ldx	#$40
	ldy	#10
	START_SAMEPAGE "write2"
wrt_lp4:
	bvc	.
	clv
	lda	emptygcr-1,y
	sta	$1c01
	dey
	bne	wrt_lp4
	ldy	#5
	dex
	bne	wrt_lp4
	END_SAMEPAGE

; write intersector gap
	lda	#$55
	ldx	fmt_intersectorgap
wrt_lp5:
	bvc	.
	clv
	sta	$1c01
	dex
	bne	wrt_lp5

; wrote sector, check if more
	dec	fmt_sectortemp
	bne	wrt_lp1

	bvc	.
	clv
	bvc	.
	clv
	jsr	$fe00	;KILL  (go to readmode)

;**************************************************************************
;*
;* Verify the track.
;*
;******
	lda	#0
	sta	$32	;HDRPNT
	lda	#200
	sta	fmt_vfyretries

	lda	$43	;SECTR
	sta	fmt_sectortemp
vfy_lp1:
; find sync
	jsr	wait_sync

; verify header block
	ldx	#10
	ldy	$32	;HDRPNT
	START_SAMEPAGE "verify1"
vfy_lp2:
	bvc	.
	clv
	lda	$1c01
	cmp	FMT_HDRBUFFER,y
	bne	vfy_fl1
	iny
	dex
	bne	vfy_lp2
	END_SAMEPAGE
	sty	$32	;HDRPNT

; find sync
	jsr	wait_sync

; verify data block
	ldx	#$40
	ldy	#10
	START_SAMEPAGE "verify2"
vfy_lp3:
	bvc	.
	clv
	lda	$1c01
	cmp	emptygcr-1,y
	bne	vfy_fl1
	dey
	bne	vfy_lp3
	ldy	#5
	dex
	bne	vfy_lp3
	END_SAMEPAGE

	dec	fmt_sectortemp
	bne	vfy_lp1

	if	MEASURE_TAIL_GAP
; X=0
	START_SAMEPAGE "gap_scan"
; count gap bytes up to the next sync
gap_lp1:
	ldy	$1c00
	bpl	gap_ex1		; sync found, skip out
	bvc	gap_lp1		; no byte sync, check sync again
	clv
	lda	$1c01
	inx
	bne	gap_lp1
	dex
; overflow, X=$ff
gap_ex1:
	END_SAMEPAGE
; adjust: unless the last byte read was $55 we counted one too many.
	cmp	#$55
	beq	gap_skp1
	dex
gap_skp1:
	txa
	ldx	fmt_zone
; handle max
	cmp	gapmax_tab,x
	bcc	gap_skp2
	sta	gapmax_tab,x
gap_skp2:
; handle min
	cmp	gapmin_tab,x
	bcs	gap_skp3
	sta	gapmin_tab,x
gap_skp3:
	if	MEASURE_TAIL_AVERAGE
	clc
	adc	gapsuml_tab,x
	sta	gapsuml_tab,x
	bcc	gap_skp4
	inc	gapsumh_tab,x
gap_skp4:	
	endif
	endif
;*
;* are we done with the disk?
;*
	lda	$51	;FTNUM
	inc	$51	;FTNUM
	cmp	conf_num_tracks
	beq	fj_ex1		;yes, exit

;*
;* no! go to job loop.
;*
	jmp	main_loop

vfy_fl1:
	dec	fmt_vfyretries
	bne	vfy_lp1
	lda	#$06	;24, 'read error'
	jmp	fj_fl1


fj_ex1:
	lda	#18
	jsr	seek_track
;*
;* format ok!
;*
	lda	#$01
	rts	; jmp $fddb


fj_fl1:
;*
;* format failure... decrease retries
;*
	dec	fmt_retries
	beq	fj_ex2
	jmp	main_loop

fj_ex2:
;*
;* set FTNUM to default value.  Set GCRFLAG so that noone will
;* try to convert from GCR to BIN after we have left.
;*
	rts	; jmp $fddb

;**************************************************************************
;*
;* writesync
;* writes 5*$ff to the disk.
;* must be run from FDC mode!
;*
;******
writesync:
	ldx	#5
	lda	#$ff
	START_SAMEPAGE "writesync"
wsy_lp1:
	bvc	.
	clv
	sta	$1c01
	dex
	bne	wsy_lp1
	END_SAMEPAGE
	rts

;**************************************************************************
;*
;* format data
;*
;******

;*
;* written:
;*     1 * emptygcr1
;*   $40 * emptygcr2
;* will result in (when GCR uncoded)
;* $07 followed by 259 $00 bytes.
;*
;* $07 is the data block identifier
;* $00 * 256 is the data bytes
;* $00 is the data block checksum
;* $00 * 2 is the off bytes.
;*
;* the GCR code is stored in reverse byte order
;*
;* emptygcr1:
;*   01010 10111 01010 01010 01010 01010 01010 01010
;*     \     /     \     /     \     /     \     /
;*       $07         $00         $00         $00
;*
;* emptygcr2: (repeat $40 times)
;*   01010 01010 01010 01010 01010 01010 01010 01010
;*     \     /     \     /     \     /     \     /
;*       $00         $00         $00         $00
;*
	START_SAMEPAGE "emptygcr"
emptygcr:
emptygcr2:
	dc.b	$4a,$29,$a5,$94,$52
emptygcr1:
	dc.b	$4a,$29,$a5,$d4,$55
	END_SAMEPAGE

;*
;* start wipeing track with $55's
;* this happens automagically once started.
;*
start_wipe:
	lda	$1c0c
	and	#$1f
	ora	#$c0
	sta	$1c0c	;set write mode
	lda	#$ff
	sta	$1c03	;port as output
	lda	#$55
	sta	$1c01	;wipe pattern
	rts

;**************************************************************************
;*
;* NAME  wait_sync
;*
;* DESCRIPTION
;*   Clone of $f556, but with custom routine on timeout.
;*
;******
wait_sync:
	lda	#$80 | $50	; 20 ms
	sta	$1805
ws_lp1:
	bit	$1805		; timeout?
	bpl	ws_fl1		; yes, fail exit
	bit	$1c00		; Sync found?
	bmi	ws_lp1		; no, try again
	lda	$1c01
	clv			; flush byte
	rts
ws_fl1:
	pla
	pla
	lda	#$03		; $03 SYNC not found
	rts	; jmp $fddb

;**************************************************************************
;*
;* NAME  seek_track, set_density
;*
;* DESCRIPTION
;*   "Seek" track "Acc" and setup density and number of sectors.
;*   Start motor if it was not already on.
;*
;******
seek_track:
	sta	track_zp
st_lp1:
	lda	track_zp
	asl			; Count half-tracks
	cmp	currhalftrk_zp
	beq	st_ex1		; currhalftrk_zp == track_zp*2
	bcc	st_skp1		; currhalftrk_zp > track_zp*2
; currhalftrk_zp < track_zp*2
	jsr	head_in
	inc	currhalftrk_zp
	jmp	st_lp1
st_skp1:
	jsr	head_out
	dec	currhalftrk_zp
	jmp	st_lp1
st_ex1:

set_density:
; set up number of sectors on track, and density
	lda	track_zp
	ldx	#3
st_lp2:
	cmp	trk_tab-1,x
	bcs	st_skp2		; in that zone
	dex
	bne	st_lp2
st_skp2:
	stx	fmt_zone
	lda	sec_tab,x
	sta	$43	; SECTR (sectors per track)
	lda	$1c00		; set density
	and	#%10011111
	ora	dens_tab,x
	sta	$1c00
	rts
trk_tab:
	dc.b	18,25,31	; the first track in each zone
sec_tab:
	dc.b	21,19,18,17

LED_STATE	equ	%00001000
dens_tab:
	dc.b	%01100000 | LED_STATE
	dc.b	%01000000 | LED_STATE
	dc.b	%00100000 | LED_STATE
	dc.b	%00000000 | LED_STATE
	
; step head outwards (= lower track num)
head_out:
	lda	#$ff
	dc.b	$2c
; step head inwards (= higher track num)
head_in:
	lda	#$01
head_common:
	clc
	adc	$1c00
	and	#%00000011
	sta	tmp_zp
	lda	$1c00	
	and	#%11110100	; mask + led off
	ora	tmp_zp
	sta	$1c00
	lda	#$80 | STEP_TIMER
delay:
	sta	$1805
dl_lp1:
	bit	$1805
	bmi	dl_lp1
	rts

;**************************************************************************
;*
;* NAME  motor_on, motor_off
;*
;* DESCRIPTION
;*   Start/stop motor.  Wait for spin-up time when going from off to on.
;*
;******
motor_on:
	lda	#%00000100	; motor on?
	bit	$1c00
	bne	mo_ex1		; yes, skip out
	ora	$1c00		; motor on
	sta	$1c00
	ldy	#SPIN_UP_TIME
mo_lp1:
	lda	#$80 | TIMER_10MS ; ~10 ms
	jsr	delay
	dey
	bne	mo_lp1
mo_ex1:
	rts

motor_off:
	lda	$1c00
	and	#%11110011	; LED off, motor off
	sta	$1c00
	rts


;**************************************************************************
;*
;* here is where we enter first.
;* started using a M-E command.
;* Now we are in IP mode.
;*
;******
format_code_entry:
;*
;* set initial formatting parameters
;*
	ldx	#$ff
	stx	$51	;FTNUM
	inx
	stx	$7f	;drive 0
	inx
; fetch the id from the bam.
; X=1
fe_lp1:
	lda	fmt_id,x
	sta	$12,x
	dex
	bpl	fe_lp1

;*
;* clear all channels
;*
	jsr	$d307	;CLRCHN

;*
;* do execute job
;*
	sei
	jsr	formatjob
	sta	last_status
	ldy	#$ff
	sty	$51
	iny
	sty	$50
	cmp	#$02
	bcc	fe_ex1
	jsr	motor_off
	cli

	lda	#$03	;21, 'read error'
	ldx	#$00
	stx	$06
	stx	$07
	jmp	$e60a	;ERROR

fe_ex1:
	cli
	if	SKIP_BAM_WRITE
	jsr	$d042		; init
; exit ok.
	jmp	$c194
	else

; LED on
	jsr	$c118

; write BAM (18,0)
	lda	#18
	sta	$0e
	lda	#0
	sta	$0f
	jsr	fe_write

; write first dir sector (18,1)
	ldx	#0
	txa
fe_lp2:
	sta	$0700,x
	inx
	bne	fe_lp2
	dec	$0701
	inc	$0f
	jsr	fe_write

; exit ok.
	jmp	$c194

fe_write:
	ldx	#10
fwr_lp1:
	lda	#$90
	sta	$04
fwr_lp2:
	ldy	$04
	bmi	fwr_lp2
	cpy	#$02
	bcc	fwr_ex1
	dex
	bne	fwr_lp1
	pla
	pla
	tya		;error code
	ldx	#$04	;Buffer 4
	jmp	$e60a	;ERROR

fwr_ex1:
	rts
	endif

;**************************************************************************
;*
;* data
;* must be filled in by master program.
;*
;******
	echo	"end",.
	ds.b	$06e4-.,$00
last_status:
	ds.b	1,$ff
last_track:
	ds.b	1,$ff
last_zone:
	ds.b	1,$ff

isgap_tab:
	ds.b	4,0
capl_tab:
	ds.b	4,0
caph_tab:
	ds.b	4,0
	if	MEASURE_TAIL_GAP
gapmax_tab:
	ds.b	4,0
gapmin_tab:
	ds.b	4,$ff
	if	MEASURE_TAIL_AVERAGE
gapsuml_tab:
	ds.b	4,0
gapsumh_tab:
	ds.b	4,0
	endif
	endif
	
	echo	"endtab",.
;* config *
	ds.b	$06fc-.,$00
conf_num_tracks:
	dc.b	FMT_TRACKS
conf_sector_gcr_space:
	dc.b	<SECTOR_GCR_SPACE

	ds.b	$0700-.,$00
;* bam *
bam_tab:
	dc.b	$12,$01,$41,$00,$15,$ff,$ff,$1f,$15,$ff,$ff,$1f,$15,$ff,$ff,$1f
	dc.b	$15,$ff,$ff,$1f,$15,$ff,$ff,$1f,$15,$ff,$ff,$1f,$15,$ff,$ff,$1f
	dc.b	$15,$ff,$ff,$1f,$15,$ff,$ff,$1f,$15,$ff,$ff,$1f,$15,$ff,$ff,$1f
	dc.b	$15,$ff,$ff,$1f,$15,$ff,$ff,$1f,$15,$ff,$ff,$1f,$15,$ff,$ff,$1f
	dc.b	$15,$ff,$ff,$1f,$15,$ff,$ff,$1f,$11,$fc,$ff,$07,$13,$ff,$ff,$07
	dc.b	$13,$ff,$ff,$07,$13,$ff,$ff,$07,$13,$ff,$ff,$07,$13,$ff,$ff,$07
	dc.b	$13,$ff,$ff,$07,$12,$ff,$ff,$03,$12,$ff,$ff,$03,$12,$ff,$ff,$03
	dc.b	$12,$ff,$ff,$03,$12,$ff,$ff,$03,$12,$ff,$ff,$03,$11,$ff,$ff,$01
	dc.b	$11,$ff,$ff,$01,$11,$ff,$ff,$01,$11,$ff,$ff,$01,$11,$ff,$ff,$01
fmt_nam:
	dc.b	$a0,$a0,$a0,$a0,$a0,$a0,$a0,$a0,$a0,$a0,$a0,$a0,$a0,$a0,$a0,$a0
fmt_id	equ	.+2
	dc.b	$a0,$a0,$a0,$a0,$a0,$32,$41,$a0,$a0,$a0,$a0,$00,$00,$00,$00,$00
	dc.b	$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
	dc.b	$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
	dc.b	$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
	dc.b	$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
	dc.b	$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00

format_code_end:
	SHOWRANGE "format drive code", format_code, format_code_end

; eof
